@@ -3,14 +3,14 @@ use strict;
 use warnings;
 use Carp qw/carp croak confess/;
 
-BEGIN {
-  $Math::Prime::Util::ECAffinePoint::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::ECAffinePoint::VERSION = '0.44_004';
+if (!defined $Math::BigInt::VERSION) {
+  eval { require Math::BigInt;   Math::BigInt->import(try=>'GMP,Pari'); 1; }
+  or do { croak "Cannot load Math::BigInt"; };
 }
 
 BEGIN {
-  do { require Math::BigInt;  Math::BigInt->import(try=>"GMP,Pari"); }
-    unless defined $Math::BigInt::VERSION;
+  $Math::Prime::Util::ECAffinePoint::AUTHORITY = 'cpan:DANAJ';
+  $Math::Prime::Util::ECAffinePoint::VERSION = '0.26';
 }
 
 # Pure perl (with Math::BigInt) manipulation of Elliptic Curves
@@ -76,57 +76,12 @@ sub _double {
   return ($x,$y);
 }
 
-sub _inplace_add {
-  my ($P1x, $P1y, $x, $y, $n) = @_;
-
-  if ($P1x == $x) {
-    my $t = ($P1y + $y) % $n;
-    if ($t == 0) {
-      ($_[2],$_[3]) = (Math::BigInt->bzero,Math::BigInt->bone);
-      return;
-    }
-  }
-  my $deltax = ($x - $P1x) % $n;
-  $deltax->bmodinv($n);
-  if ($deltax eq 'NaN') {
-    ($_[2],$_[3]) = (Math::BigInt->bzero,Math::BigInt->bone);
-    return;
-  }
-  my $deltay = ($y - $P1y) % $n;
-  my $m = ($deltay * $deltax) % $n;   # m = deltay / deltax
-
-  $_[2] = ($m*$m - $P1x - $x) % $n;
-  $_[3] = ($m*($P1x - $_[2]) - $P1y) % $n;
-}
-sub _inplace_double {
-  my($x, $y, $a, $n) = @_;
-  my $m = $y+$y;
-  $m->bmodinv($n);
-  if ($m eq 'NaN') {
-    ($_[0],$_[1]) = (Math::BigInt->bzero,Math::BigInt->bone);
-    return;
-  }
-  $m->bmul($x->copy->bmul($x)->bmul(3)->badd($a))->bmod($n);
-
-  my $xin = $x->copy;
-  $_[0] = ($m*$m - $x - $x) % $n;
-  $_[1] = ($m*($xin - $_[0]) - $y) % $n;
-}
-
 sub mul {
   my ($self, $k) = @_;
   my $x = $self->{'x'};
   my $y = $self->{'y'};
-  my $a = $self->{'a'};
   my $n = $self->{'n'};
   my $f = $self->{'f'};
-  if (ref($k) eq 'Math::BigInt' && $k < ''.~0) {
-    if ($] >= 5.008 || ~0 == 4294967295) {
-      $k = int($k->bstr);
-    } elsif ($] < 5.008 && ~0 > 4294967295 && $k < 562949953421312) {
-      $k = unpack('Q',pack('Q',$k->bstr));
-    }
-  }
 
   my $Bx = $n->copy->bzero;
   my $By = $n->copy->bone;
@@ -136,16 +91,16 @@ sub mul {
     if ( ($k % 2) != 0) {
       $k--;
       $f->bmul($Bx-$x)->bmod($n);
-      if    ($x->is_zero && $y->is_one)   { }
-      elsif ($Bx->is_zero && $By->is_one) { ($Bx,$By) = ($x,$y); }
-      else                                { _inplace_add($x,$y,$Bx,$By,$n); }
+      if    ($x == 0 && $y == 1)   { }
+      elsif ($Bx == 0 && $By == 1) { ($Bx,$By) = ($x,$y); }
+      else                         { ($Bx,$By) = $self->_add($x,$y,$Bx,$By); }
     } else {
       $k >>= 1;
       $f->bmul(2*$y)->bmod($n);
-      _inplace_double($x,$y,$a,$n);
+      ($x,$y) = $self->_double($x,$y);
     }
   }
-  $f = Math::BigInt::bgcd($f, $n);
+  $f = Math::BigInt::bgcd( $f, $n);
   $self->{'x'} = $Bx;
   $self->{'y'} = $By;
   $self->{'f'} = $f;
@@ -165,7 +120,7 @@ sub add {
                                              $other->{'x'}, $other->{'y'});
   return $self;
 }
-
+  
 
 sub a { return shift->{'a'}; }
 sub b { return shift->{'b'}; }
@@ -190,10 +145,6 @@ __END__
 
 =encoding utf8
 
-=for stopwords mul
-
-=for test_synopsis use v5.14;  my($a,$b,$n,$k,$ECP2);
-
 
 =head1 NAME
 
@@ -202,7 +153,7 @@ Math::Prime::Util::ECAffinePoint - Elliptic curve operations for affine points
 
 =head1 VERSION
 
-Version 0.44_004
+Version 0.26
 
 
 =head1 SYNOPSIS
@@ -211,10 +162,10 @@ Version 0.44_004
   my $ECP = Math::Prime::Util::ECAffinePoint->new($a, $b, $n, 0, 1);
 
   # scalar multiplication by k.
-  $ECP->mul($k);
+  $ECP->mul($k)
 
   # add two points on the same curve
-  $ECP->add($ECP2);
+  $ECP->add($ECP2)
 
   say "P = O" if $ECP->is_infinity();
 
