@@ -3,14 +3,14 @@ use strict;
 use warnings;
 use Carp qw/carp croak confess/;
 
-BEGIN {
-  $Math::Prime::Util::ECProjectivePoint::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::ECProjectivePoint::VERSION = '0.44_004';
+if (!defined $Math::BigInt::VERSION) {
+  eval { require Math::BigInt;   Math::BigInt->import(try=>'GMP,Pari'); 1; }
+  or do { croak "Cannot load Math::BigInt"; };
 }
 
 BEGIN {
-  do { require Math::BigInt;  Math::BigInt->import(try=>"GMP,Pari"); }
-    unless defined $Math::BigInt::VERSION;
+  $Math::Prime::Util::ECProjectivePoint::AUTHORITY = 'cpan:DANAJ';
+  $Math::Prime::Util::ECProjectivePoint::VERSION = '0.26';
 }
 
 # Pure perl (with Math::BigInt) manipulation of Elliptic Curves
@@ -38,7 +38,7 @@ sub new {
   bless $self, $class;
   return $self;
 }
-
+ 
 sub _addx {
   my ($x1, $x2, $xin, $n) = @_;
 
@@ -57,26 +57,20 @@ sub _add3 {
   my $u = ($x2 - $z2) * ($x1 + $z1);
   my $v = ($x2 + $z2) * ($x1 - $z1);
 
-  my $upv2 = $u + $v;  $upv2->bmul($upv2);
-  my $umv2 = $u - $v;  $umv2->bmul($umv2);
+  my $upv2 = ($u + $v) ** 2;
+  my $umv2 = ($u - $v) ** 2;
 
-  $upv2->bmul($zin)->bmod($n);
-  $umv2->bmul($xin)->bmod($n);
-  return ($upv2, $umv2);
+  return ( ($upv2*$zin) % $n, ($umv2*$xin) % $n );
 }
 
 sub _double {
   my ($x, $z, $n, $d) = @_;
 
-  my $u = $x + $z;   $u->bmul($u);
-  my $v = $x - $z;   $v->bmul($v);
-
+  my $u = ($x + $z) ** 2;
+  my $v = ($x - $z) ** 2;
   my $w = $u - $v;
   my $t = $d * $w + $v;
-
-  $u->bmul($v)->bmod($n);
-  $w->bmul($t)->bmod($n);
-  return ($u, $w);
+  return ( ($u * $v) % $n  ,  ($w * $t) % $n );
 }
 
 sub mul {
@@ -197,10 +191,6 @@ __END__
 
 =encoding utf8
 
-=for stopwords mul
-
-=for test_synopsis use v5.14;  my($c,$n,$k,$ECP2);
-
 
 =head1 NAME
 
@@ -209,7 +199,7 @@ Math::Prime::Util::ECProjectivePoint - Elliptic curve operations for projective
 
 =head1 VERSION
 
-Version 0.44_004
+Version 0.26
 
 
 =head1 SYNOPSIS
@@ -217,11 +207,11 @@ Version 0.44_004
   # Create a point on a curve (a,b,n) with coordinates 0,1
   my $ECP = Math::Prime::Util::ECProjectivePoint->new($c, $n, 0, 1);
 
-  # scalar multiplication by $k.
-  $ECP->mul($k);
+  # scalar multiplication by k.
+  $ECP->mul($k)
 
   # add two points on the same curve
-  $ECP->add($ECP2);
+  $ECP->add($ECP2)
 
   say "P = O" if $ECP->is_infinity();
 
@@ -283,7 +273,7 @@ Returns a copy of the point.
 
 =head2 normalize
 
-Performs an extended GCD operation to make C<z=1>.  If a factor of C<n> is
+Performs an extended gcd operation to make C<z=1>.  If a factor of C<n> is
 found it is put in C<f>.
 
 
